package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.Film;
import my.test.sakila.model.Language;

/**
 * Backing bean for Film entities.
 * <p>
 * This class provides CRUD functionality for all Film entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class FilmBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving Film entities
	 */

	private Short id;

	public Short getId() {
		return this.id;
	}

	public void setId(Short id) {
		this.id = id;
	}

	private Film film;

	public Film getFilm() {
		return this.film;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.film = this.search;
		} else {
			this.film = this.entityManager.find(Film.class, getId());
		}
	}

	/*
	 * Support updating and deleting Film entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.film);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.film);
				return "view?faces-redirect=true&id=" + this.film.getFilmId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(Film.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching Film entities with pagination
	 */

	private int page;
	private long count;
	private List<Film> pageItems;
	
	private Film search = new Film();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public Film getSearch() {
		return this.search;
	}

	public void setSearch(Film search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<Film> root = countCriteria.from(Film.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<Film> criteria = builder.createQuery(Film.class);
		root = criteria.from(Film.class);
		TypedQuery<Film> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<Film> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		Language languageByOriginalLanguageId = this.search.getLanguageByOriginalLanguageId();
		if (languageByOriginalLanguageId != null && languageByOriginalLanguageId.getLanguageId() != null) {
			predicatesList.add(builder.equal(root.get("languageByOriginalLanguageId"), languageByOriginalLanguageId));
		}
		Language languageByLanguageId = this.search.getLanguageByLanguageId();
		if (languageByLanguageId != null && languageByLanguageId.getLanguageId() != null) {
			predicatesList.add(builder.equal(root.get("languageByLanguageId"), languageByLanguageId));
		}
		String title = this.search.getTitle();
		if (title != null && !"".equals(title)) {
			predicatesList.add(builder.like(root.<String>get("title"), '%' + title + '%'));
		}
		String description = this.search.getDescription();
		if (description != null && !"".equals(description)) {
			predicatesList.add(builder.like(root.<String>get("description"), '%' + description + '%'));
		}
		String rating = this.search.getRating();
		if (rating != null && !"".equals(rating)) {
			predicatesList.add(builder.like(root.<String>get("rating"), '%' + rating + '%'));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<Film> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back Film entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<Film> getAll() {

		CriteriaQuery<Film> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(Film.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Film.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return FilmBean.this.entityManager.find(Film.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((Film) value).getFilmId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private Film add = new Film();

	public Film getAdd() {
		return this.add;
	}

	public Film getAdded() {
		Film added = this.add;
		this.add = new Film();
		return added;
	}
}