package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.Inventory;
import my.test.sakila.model.Film;
import my.test.sakila.model.Store;

/**
 * Backing bean for Inventory entities.
 * <p>
 * This class provides CRUD functionality for all Inventory entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class InventoryBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving Inventory entities
	 */

	private Integer id;

	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	private Inventory inventory;

	public Inventory getInventory() {
		return this.inventory;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.inventory = this.search;
		} else {
			this.inventory = this.entityManager.find(Inventory.class, getId());
		}
	}

	/*
	 * Support updating and deleting Inventory entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.inventory);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.inventory);
				return "view?faces-redirect=true&id=" + this.inventory.getInventoryId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(Inventory.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching Inventory entities with pagination
	 */

	private int page;
	private long count;
	private List<Inventory> pageItems;
	
	private Inventory search = new Inventory();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public Inventory getSearch() {
		return this.search;
	}

	public void setSearch(Inventory search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<Inventory> root = countCriteria.from(Inventory.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<Inventory> criteria = builder.createQuery(Inventory.class);
		root = criteria.from(Inventory.class);
		TypedQuery<Inventory> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<Inventory> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		int inventoryId = this.search.getInventoryId();
		if (inventoryId != 0) {
			predicatesList.add(builder.equal(root.get("inventoryId"), inventoryId));
		}
		Store store = this.search.getStore();
		if (store != null && store.getStoreId() != null) {
			predicatesList.add(builder.equal(root.get("store"), store));
		}
		Film film = this.search.getFilm();
		if (film != null && film.getFilmId() != null) {
			predicatesList.add(builder.equal(root.get("film"), film));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<Inventory> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back Inventory entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<Inventory> getAll() {

		CriteriaQuery<Inventory> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(Inventory.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Inventory.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return InventoryBean.this.entityManager.find(Inventory.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((Inventory) value).getInventoryId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private Inventory add = new Inventory();

	public Inventory getAdd() {
		return this.add;
	}

	public Inventory getAdded() {
		Inventory added = this.add;
		this.add = new Inventory();
		return added;
	}
}