package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.Rental;
import my.test.sakila.model.Customer;
import my.test.sakila.model.Inventory;
import my.test.sakila.model.Staff;

/**
 * Backing bean for Rental entities.
 * <p>
 * This class provides CRUD functionality for all Rental entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class RentalBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving Rental entities
	 */

	private Integer id;

	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	private Rental rental;

	public Rental getRental() {
		return this.rental;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.rental = this.search;
		} else {
			this.rental = this.entityManager.find(Rental.class, getId());
		}
	}

	/*
	 * Support updating and deleting Rental entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.rental);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.rental);
				return "view?faces-redirect=true&id=" + this.rental.getRentalId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(Rental.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching Rental entities with pagination
	 */

	private int page;
	private long count;
	private List<Rental> pageItems;
	
	private Rental search = new Rental();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public Rental getSearch() {
		return this.search;
	}

	public void setSearch(Rental search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<Rental> root = countCriteria.from(Rental.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<Rental> criteria = builder.createQuery(Rental.class);
		root = criteria.from(Rental.class);
		TypedQuery<Rental> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<Rental> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		int rentalId = this.search.getRentalId();
		if (rentalId != 0) {
			predicatesList.add(builder.equal(root.get("rentalId"), rentalId));
		}
		Staff staff = this.search.getStaff();
		if (staff != null && staff.getStaffId() != null) {
			predicatesList.add(builder.equal(root.get("staff"), staff));
		}
		Customer customer = this.search.getCustomer();
		if (customer != null && customer.getCustomerId() != null) {
			predicatesList.add(builder.equal(root.get("customer"), customer));
		}
		Inventory inventory = this.search.getInventory();
		if (inventory != null && inventory.getInventoryId() != null) {
			predicatesList.add(builder.equal(root.get("inventory"), inventory));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<Rental> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back Rental entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<Rental> getAll() {

		CriteriaQuery<Rental> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(Rental.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Rental.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return RentalBean.this.entityManager.find(Rental.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((Rental) value).getRentalId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private Rental add = new Rental();

	public Rental getAdd() {
		return this.add;
	}

	public Rental getAdded() {
		Rental added = this.add;
		this.add = new Rental();
		return added;
	}
}