package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.Staff;
import my.test.sakila.model.Address;
import my.test.sakila.model.Store;

/**
 * Backing bean for Staff entities.
 * <p>
 * This class provides CRUD functionality for all Staff entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class StaffBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving Staff entities
	 */

	private Byte id;

	public Byte getId() {
		return this.id;
	}

	public void setId(Byte id) {
		this.id = id;
	}

	private Staff staff;

	public Staff getStaff() {
		return this.staff;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.staff = this.search;
		} else {
			this.staff = this.entityManager.find(Staff.class, getId());
		}
	}

	/*
	 * Support updating and deleting Staff entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.staff);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.staff);
				return "view?faces-redirect=true&id=" + this.staff.getStaffId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(Staff.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching Staff entities with pagination
	 */

	private int page;
	private long count;
	private List<Staff> pageItems;
	
	private Staff search = new Staff();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public Staff getSearch() {
		return this.search;
	}

	public void setSearch(Staff search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<Staff> root = countCriteria.from(Staff.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<Staff> criteria = builder.createQuery(Staff.class);
		root = criteria.from(Staff.class);
		TypedQuery<Staff> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<Staff> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		Address address = this.search.getAddress();
		if (address != null && address.getAddressId() != null) {
			predicatesList.add(builder.equal(root.get("address"), address));
		}
		Store store = this.search.getStore();
		if (store != null && store.getStoreId() != null) {
			predicatesList.add(builder.equal(root.get("store"), store));
		}
		String firstName = this.search.getFirstName();
		if (firstName != null && !"".equals(firstName)) {
			predicatesList.add(builder.like(root.<String>get("firstName"), '%' + firstName + '%'));
		}
		String lastName = this.search.getLastName();
		if (lastName != null && !"".equals(lastName)) {
			predicatesList.add(builder.like(root.<String>get("lastName"), '%' + lastName + '%'));
		}
		String email = this.search.getEmail();
		if (email != null && !"".equals(email)) {
			predicatesList.add(builder.like(root.<String>get("email"), '%' + email + '%'));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<Staff> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back Staff entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<Staff> getAll() {

		CriteriaQuery<Staff> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(Staff.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Staff.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return StaffBean.this.entityManager.find(Staff.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((Staff) value).getStaffId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private Staff add = new Staff();

	public Staff getAdd() {
		return this.add;
	}

	public Staff getAdded() {
		Staff added = this.add;
		this.add = new Staff();
		return added;
	}
}